/**
 * @file        General image component
 * @author      MicroMatrix <yangpan.1985@bytedance.com>
 * @copyright   2012-2020 bytedance
 * @link        git@code.byted.org:yangpan.1985/avg.git
 */

import componentify from '../pixi/componentify';
import { mountNode, updateNode, setValue, updateValue } from '../pixi/properties';
import RichTextNode from 'pixi-richtext';

export const RichText =  componentify('RichText', {

  createNode() {
    this.node = new RichTextNode();
  },
  mountNode(props) {
    const node = this.node;

    setValue.call(node, 'style', props.style);
    setValue.call(node, 'layout', props.layout);
    setValue.call(node, 'text', props.text);
    mountNode(node, props);

    return node;
  },
  updateNode(prevProps, props) {
    const node = this.node;

    updateValue.call(node, 'style', prevProps.style, props.style);
    updateValue.call(node, 'layout', prevProps.layout, props.layout);
    updateValue.call(node, 'text', prevProps.text, props.text);
    updateNode(node, prevProps, props);
  },

});
